@@ -55,13 +55,10 @@ echo -n .
 "${ZCAT}" < in.gz > copy || fail=1
 cmp in copy || fail=1
 echo -n .
-"${ZCAT}" --gzip < in.gz > copy || fail=1
+"${ZCAT}" < in.bz2 > copy || fail=1
 cmp in copy || fail=1
 echo -n .
-"${ZCAT}" --bzip2 < in.bz2 > copy || fail=1
-cmp in copy || fail=1
-echo -n .
-"${ZCAT}" --lzip < in.lz > copy || fail=1
+"${ZCAT}" < in.lz > copy || fail=1
 cmp in copy || fail=1
 echo -n .
 "${ZCAT}" in > copy || fail=1
@@ -122,11 +119,9 @@ done
 echo -n .
 "${ZGREP}" License < in.gz 2>&1 > /dev/null || fail=1
 echo -n .
-"${ZGREP}" --gzip License < in.gz 2>&1 > /dev/null || fail=1
+"${ZGREP}" License < in.bz2 2>&1 > /dev/null || fail=1
 echo -n .
-"${ZGREP}" --bzip2 License < in.bz2 2>&1 > /dev/null || fail=1
-echo -n .
-"${ZGREP}" --lzip License < in.lz 2>&1 > /dev/null || fail=1
+"${ZGREP}" License < in.lz 2>&1 > /dev/null || fail=1
 echo -n .
 "${ZGREP}" License in 2>&1 > /dev/null || fail=1
 echo -n .
